PlayheadBehavior

Struct PlayheadBehavior 

pub struct PlayheadBehavior {
    pub playback_speed: f64,
    pub is_playing_backwards: bool,
}
Information a SampleResource can use to make optimal caching decisions based on the behavior of the playhead.

Fields

playback_speed: f64

The speed at which the playhead is moving. (Where 1.0 is playing at the sample rate of this resource, 0.5 is playing at half the sample rate of this resource, and 2.0 is playing at twice the speed of the sample rate of this resource).

is_playing_backwards: bool

True if playing backwards, false if playing forwards.