SampleResourceInfo

Struct SampleResourceInfo 

pub struct SampleResourceInfo {
    pub num_channels: usize,
    pub len_frames: u64,
    pub sample_rate: Option<f64>,
}
Information about a SampleResource or a SampleResourceF32, a resource of audio samples with a known length.

This information is NOT allowed to change for the lifetime of this sample resource.

Fields

num_channels: usize

The number of audio channels in this resource.

len_frames: u64

The length of this resource in frames (samples of a single channel of audio, not to be confused with video frames).

sample_rate: Option<f64>

The sample rate of this resource. This will beNone if the sample rate is unknown.